<?php

namespace Sale\Handlers\PaySystem;

use Bitrix\Main\Error;
use Bitrix\Main\Localization\Loc;
use Bitrix\Main\Request;
use Bitrix\Main\Text\Encoding;
use Bitrix\Main\Type\DateTime;
use Bitrix\Main\Web;
use Bitrix\Sale\BusinessValue;
use Bitrix\Sale\PaySystem;
use Bitrix\Sale\Payment;
use Bitrix\Sale\Order;
use Bitrix\Main\Loader;
use Bitrix\Main\Config\Option;

IncludeModuleLangFile(__FILE__);
require_once dirname(dirname(__FILE__)) . '/config.php';
define('DEF_MODULE_ID', $RBS_CONFIG['MODULE_ID']);
Loader::includeModule( 'rbs.payment' );

/**
 * Class rbsPaymentHandler
 * @package Sale\Handlers\PaySystem
 */
class rbs_paymentHandler extends PaySystem\ServiceHandler implements PaySystem\IPrePayable
{
	/**
	 * @param Payment $payment
	 * @param Request|null $request
	 * @return PaySystem\ServiceResult
	 */
	public function initiatePay(Payment $payment, Request $request = null)
	{

		$moduleId = 'rbs.payment';
		
		$RBS_Gateway = new \Rbs\Payments\Gateway;

		
		// module settings
		$RBS_Gateway->setOptions(array(
			'module_id' => Option::get($moduleId, 'MODULE_ID'),
			'gate_url_prod' => Option::get($moduleId, 'RBS_PROD_URL'),
			'gate_url_test' => Option::get($moduleId, 'RBS_TEST_URL'),
			'module_version' => Option::get($moduleId, 'MODULE_VERSION'),
			'iso' => unserialize(Option::get($moduleId, 'ISO')),
			'cms_version' => 'Bitrix ' . SM_VERSION,
			'language' => 'ru',
		));

		// handler settings
		$RBS_Gateway->setOptions(array(
			'ofd_tax' => $this->getBusinessValue($payment, 'OFD_TAX_SYSTEM') == 0 ? 0 : $this->getBusinessValue($payment, 'OFD_TAX_SYSTEM'),
			'ofd_enabled' => $this->getBusinessValue($payment, 'OFD_RECIEPT')  == 'Y' ? 1 : 0,
			'ffd_version' => $this->getBusinessValue($payment, 'FFD_VERSION'),
			'ffd_payment_object' => $this->getBusinessValue($payment, 'FFD_PAYMENT_OBJECT'),
			'ffd_payment_method' => $this->getBusinessValue($payment, 'FFD_PAYMENT_METHOD'),
			'test_mode' => $this->getBusinessValue($payment, 'API_TEST_MODE') == 'Y' ? 1 : 0,
			'handler_logging' => $this->getBusinessValue($payment, 'HANDLER_LOGGING') == 'Y' ? 1 : 0,
			'handler_two_stage' => $this->getBusinessValue($payment, 'HANDLER_TWO_STAGE') == 'Y' ? 1 : 0,
		));

		$RBS_Gateway->buildData(array(
			'orderNumber' => $this->getBusinessValue($payment, 'RBS_ORDER_NUMBER'),
		    'amount' => $this->getBusinessValue($payment, 'RBS_ORDER_AMOUNT'),
		    'userName' => $this->getBusinessValue($payment, 'API_LOGIN'),
		    'password' => $this->getBusinessValue($payment, 'API_PASSWORD'),
		    'currency' => $RBS_Gateway->getCurrencyCode( $payment->getField('CURRENCY') ),
		    'description' => $this->getBusinessValue($payment, 'RBS_ORDER_DESCRIPTION'),
		));

		$protocol = isset($_SERVER['HTTPS']) && $_SERVER['HTTPS'] !== "off" ? 'https://' : 'http://';
		$RBS_Gateway->buildData(array(
		    'returnUrl' => $protocol . $_SERVER['SERVER_NAME'] . '/bank/result.php' . '?PAYMENT=RBS&ORDER_ID=' . $payment->getField('ORDER_ID') . '&PAYMENT_ID=' . $payment->getField('ID')
		));

		if ($RBS_Gateway->ofdEnable()) {

			$Order = Order::load($payment->getOrderId());
			$propertyCollection = $Order->getPropertyCollection();
			$Basket = $Order->getBasket();
			$basketItems = $Basket->getBasketItems();

			$RBS_Gateway->setOptions(array(
				'customer_name' => $this->getPropertyValueByCode($propertyCollection, 'FIO'),
				'customer_email' => $this->getPropertyValueByCode($propertyCollection, 'EMAIL'),
			));

			$lastIndex = 0;
			foreach ($basketItems as $key => $BasketItem) {
				$lastIndex = $key + 1;
		        $RBS_Gateway->setPosition(array(
		            'positionId' => $key,
		            'itemCode' => $BasketItem->getProductId(),
		            'name' => $BasketItem->getField('NAME'),
		            'itemAmount' => $BasketItem->getFinalPrice(),
		            'itemPrice' => $BasketItem->getPrice(),
		            'quantity' => array(
		                'value' => $BasketItem->getQuantity(),
		                'measure' => $BasketItem->getField('MEASURE_NAME'),
		            ),
		            'tax' => array(
		                'taxType' =>  $RBS_Gateway->getTaxCode( $BasketItem->getField('VAT_RATE') * 100 ),
		            ),
		        ));
			}

			if($Order->getField('PRICE_DELIVERY') > 0) {
				
				Loader::includeModule('catalog');
				$deliveryInfo = \Bitrix\Sale\Delivery\Services\Manager::getById($Order->getField('DELIVERY_ID'));

				$deliveryVatItem = \CCatalogVat::GetByID($deliveryInfo['VAT_ID'])->Fetch();
				$RBS_Gateway->setOptions(array(
				    'delivery' => true,
				));
				$RBS_Gateway->setPosition(array(
		            'positionId' => $lastIndex + 1,
		            'itemCode' => 'DELIVERY_' . $Order->getField('DELIVERY_ID'),
		            'name' => Loc::getMessage('RBS_FIRLD_DELIVERY'),
		            'itemAmount' => $Order->getField('PRICE_DELIVERY'),
		            'itemPrice' => $Order->getField('PRICE_DELIVERY'),
		            'quantity' => array(
		                'value' => 1,
		                'measure' => Loc::getMessage('RBS_FIELD_MEASURE'),
		            ),
		            'tax' => array(
		                'taxType' => $RBS_Gateway->getTaxCode($deliveryVatItem['RATE']),
		            ),
		        ));	
			}
		}

		$gateResponse = $RBS_Gateway->registerOrder();

		$params = array(
	        'rbs_result' => $gateResponse,
	        'payment_link' => $RBS_Gateway->getPaymentLink(),
	        'currency' => $payment->getField('CURRENCY')
	    );
	    $this->setExtraParams($params);

	    return $this->showTemplate($payment, "payment");
	}

	public function processRequest(Payment $payment, Request $request)
	{
		global $APPLICATION;
		$moduleId = 'rbs.payment';

		$RBS_Gateway = new \Rbs\Payments\Gateway;
		$RBS_Gateway->setOptions(array(
			// module settings
			'gate_url_prod' => Option::get($moduleId, 'RBS_PROD_URL'),
			'gate_url_test' => Option::get($moduleId, 'RBS_TEST_URL'),
			'test_mode' => $this->getBusinessValue($payment, 'API_TEST_MODE') == 'Y' ? 1 : 0,
		));

		$RBS_Gateway->buildData(array(
		    'userName' => $this->getBusinessValue($payment, 'API_LOGIN'),
		    'password' => $this->getBusinessValue($payment, 'API_PASSWORD'),
		    'orderId' => $request->get('orderId'),
		));
		
		$gateResponse = $RBS_Gateway->checkOrder();

		$resultId = explode("_", $gateResponse['orderNumber'] );
        array_pop($resultId);
        $resultId = implode('_', $resultId);

        $successPayment = true;
        
        if($resultId != $this->getBusinessValue($payment, 'RBS_ORDER_NUMBER')) {
			$successPayment = false;
		}
        if( $gateResponse['errorCode'] != 0 || ($gateResponse['orderStatus'] != 1 && $gateResponse['orderStatus'] != 2) ) {
        	$successPayment = false;
        }
        

        if($successPayment && !$payment->isPaid()) {

        	// set payment status
        	$order = Order::load($payment->getOrderId());
			$paymentCollection = $order->getPaymentCollection();
			
			foreach ($paymentCollection as $col_payment) {
				if($col_payment->getField('ID') == $payment->getField('ID')) {
					$col_payment->setPaid("Y");
					$col_payment->setFields(array(
		                "PS_SUM" => $gateResponse["amount"] / 100,
		                "PS_CURRENCY" => $gateResponse["currency"],
		                "PS_RESPONSE_DATE" => new DateTime(),
		                "PS_STATUS" => "Y",
		                "PS_STATUS_DESCRIPTION" => $gateResponse["cardAuthInfo"]["pan"] . ";" . $gateResponse['cardAuthInfo']["cardholderName"],
		                "PS_STATUS_MESSAGE" => $gateResponse["paymentAmountInfo"]["paymentState"],
		                "PS_STATUS_CODE" =>  $gateResponse['orderStatus'],
	        		));

	        		break;
				}
			}

			if($order->isPaid()) {
				// // set order status
				$order->setField('STATUS_ID', Option::get($moduleId, 'RESULT_ORDER_STATUS'));

				// set delivery status
				if($this->getBusinessValue($payment, 'HANDLER_SHIPMENT') == 'Y') {
					$shipmentCollection = $order->getShipmentCollection();
					foreach ($shipmentCollection as $shipment){
					    if (!$shipment->isSystem()) {
			        		$shipment->allowDelivery();
					    }
			    	}
		    	}
	    	}

		    $order->save();
        }

        if($successPayment) {
        	$APPLICATION->SetTitle(Loc::getMessage('RBS_MESSAGE_THANKS'));
        	echo Loc::getMessage('RBS_MESSAGE_THANKS_DESCRIPTION') . $payment->getField('ORDER_ID');
        } else {
        	$APPLICATION->SetTitle(Loc::getMessage('RBS_MESSAGE_ERROR'));
        	echo Loc::getMessage('RBS_MESSAGE_ERROR') . ' #' . $payment->getField('ORDER_ID');
        }
       
        return new PaySystem\ServiceResult();
	}

	public function getPaymentIdFromRequest(Request $request)
	{
	    $paymentId = $request->get('PAYMENT_ID');
	    return intval($paymentId);
	}

	public function getCurrencyList()
	{
		return 'RUB';
	}

	public static function getIndicativeFields()
	{
		return array('PAYMENT' => 'RBS');
	}

	static protected function isMyResponseExtended(Request $request, $paySystemId)
	{
		return true;
	}

	private function getPropertyValueByCode($propertyCollection, $code) {
		$property = '';
		foreach ($propertyCollection as $property)
	    {
	        if($property->getField('CODE') == $code)
	            return $property->getValue();
	    }
	}


	/**
	 * @return array
	 */
	protected function getUrlList()
	{
		return array(

		);
	}
	/**
	 * @return array
	 */
	public function getProps()
	{
		$data = array();

		return $data;
	}
	/**
	 * @param Payment $payment
	 * @param Request $request
	 * @return bool
	 */
	public function initPrePayment(Payment $payment = null, Request $request)
	{
		return true;
	}
	/**
	 * @param array $orderData
	 */
	public function payOrder($orderData = array())
	{

	}
	/**
	 * @param array $orderData
	 * @return bool|string
	 */
	public function BasketButtonAction($orderData = array())
	{
		return true;
	}
	/**
	 * @param array $orderData
	 */
	public function setOrderConfig($orderData = array())
	{
		if ($orderData)
			$this->prePaymentSetting = array_merge($this->prePaymentSetting, $orderData);
	}
	public function isTuned(){}
	public function isRefundableExtended(){}
	public function confirm(Payment $payment){}
	public function cancel(Payment $payment){}
	public function refund(Payment $payment, $refundableSum){}
	public function sendResponse(PaySystem\ServiceResult $result, Request $request){}

}